@@ -42,7 +42,7 @@ import tempfile
 import shutil
 
 
-__version__ = "0.1.1"
+__version__ = "0.2"
 
 
 @contextlib.contextmanager
@@ -70,7 +70,7 @@ class Command(object):
             os.system(cmd)
 
 
-_TEX_TEMPLATE = r'''\documentclass[%sletterpaper,dvips]{article}
+_TEX_TEMPLATE = r'''\documentclass[%(options)sletterpaper,dvips]{article}
 \usepackage[usenames]{color}
 \usepackage{pst-all}
 \usepackage{pst-3dplot}
@@ -78,80 +78,115 @@ _TEX_TEMPLATE = r'''\documentclass[%sletterpaper,dvips]{article}
 \usepackage{pst-coil}
 \usepackage{pst-bar}
 \usepackage{multido}
-\usepackage{cmbright}
 \usepackage{fancyvrb}
+%(packages)s
 \begin{document}
 \pagestyle{empty}
 \begin{TeXtoEPS}
-%s
+%(code)s
 \end{TeXtoEPS}
 \end{document}'''
 
 
 class Pstricks(object):
-    def __init__(self, verbosity=3, keep_tmp=False, **kw):
+    def __init__(self, verbosity=3, **kw):
         self.verbosity = verbosity
-        self.keep_tmp = keep_tmp
-        self.tmp_main = 'makeeps_tmp'
         self.tex_template = _TEX_TEMPLATE
         self.cmd_latex = Command("latex")
         self.cmd_dvips = Command("dvips")
         self.cmd_convert = Command("convert")
 
-    def pst(self, src, dst, options=None):
+    def write_tex(self, src, dst, cmbright=False, options=None, packages=None):
         # Sanitize options.
         options = ','.join([] if None is options else options)
         if options:
             options += ','
+        # Sanitize packages.
+        packages = [] if None is packages else packages
+        if cmbright:
+            cmbright = "\usepackage{cmbright}"
+            if not packages or cmbright not in packages:
+                packages.insert(0, cmbright)
+        packages = '\n'.join(packages)
         # Prepare TeX data.
-        tex = self.tex_template % (options, open(src).read())
-        tmpf = open(self.tmp_main+'.tex', 'w').write(tex)
-        self.cmd_latex(self.tmp_main+'.tex')
-        self.cmd_dvips(self.tmp_main+'.dvi', '-E -o %s' % dst)
-        if not self.keep_tmp:
-            for fn in glob.glob(self.tmp_main+'.*'):
-                os.unlink(fn)
+        data = dict(options=options, packages=packages, code=open(src).read())
+        tex = self.tex_template % data
+        # Write the TeX data to the destination file.
+        with open(dst, 'w') as tmpf:
+            tmpf.write(tex)
+
+    def pst(self, srcmain, dst):
+        self.cmd_latex(srcmain+'.tex')
+        self.cmd_dvips(srcmain+'.dvi', '-E -o %s' % dst)
 
     def imconvert(self, src, dst, dpi=300):
         self.cmd_convert('-density %d -units PixelsPerInch %s %s' % (
             dpi, src, dst))
 
-    def __call__(self, fn):
+    def __call__(self, fn, cmbright=None, keep_tmp=None, **kw):
         with _remember_cwd():
             os.chdir(fn.tempdir)
             sys.stdout.write("Working in %s ...\n" % os.getcwd())
+            cmbright = "cmbright" == cmbright
+            self.write_tex(fn.sourcepath, fn.intertex, cmbright=cmbright)
+            self.pst(fn.destmain, fn.intereps)
             if 'eps' == fn.ftype:
-                self.pst(fn.sourcepath, fn.intereps)
                 shutil.copyfile(fn.intereps, fn.destpath)
             elif 'png' == fn.ftype:
-                self.pst(fn.sourcepath, fn.intereps)
                 self.imconvert(fn.intereps, fn.destpath)
-                if self.keep_tmp:
-                    shutil.copyfile(fn.intereps,
-                                    os.path.join(fn.destdirabs, fn.intereps))
             else:
                 sys.stdout.write("Destination file type %s isn't supported.\n" %
                                  dstext)
-        sys.stdout.write("Removing the temporary working directory %s ... " % fn.tempdir)
-        shutil.rmtree(fn.tempdir)
-        sys.stdout.write("Done\n")
+        tempdir = os.path.abspath(fn.tempdir)
+        if not keep_tmp:
+            sys.stdout.write(
+                "Removing the temporary working directory %s ... " % tempdir)
+            shutil.rmtree(fn.tempdir)
+            sys.stdout.write("Done\n")
+        else:
+            sys.stdout.write(
+                "Temporary files were kept in the directory %s\n" % tempdir)
 
 
 class Filename(object):
-    def __init__(self, source, dest, ftype=''):
+    def __init__(self, source, dest='', ftype='', tempdir=None):
         """
+        This class is responsible for determining the file names to be used in
+        :py:class:`Pstricks`.
+
+        One should specify either the *dest* argument with an extension name or
+        the *ftype* argument.
+
+        >>> # Good.
         >>> Filename(source='dir1/main.tex', dest='dir2/', ftype='png')
         Filename(source='dir1/main.tex', dest='dir2/main.png')
-        >>> Filename(source='dir1/main.tex', dest='dir2/other.png')
-        Filename(source='dir1/main.tex', dest='dir2/other.png')
-        >>> Filename(source='dir1/main.tex', dest='dir2/other.png', ftype='png')
+        >>> # Good.
+        >>> Filename(source='dir1/main.tex', dest='other.png')
+        Filename(source='dir1/main.tex', dest='other.png')
+        >>> # Good.
+        >>> Filename(source='dir1/main.tex', dest='', ftype='png')
+        Filename(source='dir1/main.tex', dest='main.png')
+
+        Construction should fail if the extension name of the destination file
+        can't be determined:
+
+        >>> Filename(source='dir1/main.tex', dest='other')
         Traceback (most recent call last):
             ...
-        ValueError: can't set both dest and ftype
-        >>> Filename(source='dir1/main.tex', dest='dir2/')
+        ValueError: can't determine the destination file type
+        >>> Filename(source='dir1/main.tex')
         Traceback (most recent call last):
             ...
         ValueError: either dest or ftype needs to be specified
+
+        If both *dest* and *ftype* are given, the extension name of the
+        destination file is deduced from *dest* if possible.  If *dest* doesn't
+        contain the extension name, use *ftype*.
+
+        >>> Filename(source='dir1/main.tex', dest='dir2/other.png', ftype='eps')
+        Filename(source='dir1/main.tex', dest='dir2/other.png')
+        >>> Filename(source='dir1/main.tex', dest='dir2/other', ftype='eps')
+        Filename(source='dir1/main.tex', dest='dir2/other.eps')
         """
         # Sanitize source file name.
         if not source.lower().endswith(".tex"):
@@ -166,30 +201,42 @@ class Filename(object):
             destdir, destbase = os.path.split(dest)
         else:
             destdir = destbase = ''
+        if not destdir and os.path.isdir(destbase):
+            destdir, destbase = destbase, ''
         destdirabs = os.path.abspath(destdir)
         # Compute destination file name.
         if destbase:
-            if ftype:
-                raise ValueError('can\'t set both dest and ftype')
             destmain, destext = os.path.splitext(destbase)
             destext = destext.strip('.')
+            destext = destext if destext else ftype
         elif ftype:
             destmain, destext = sourcemain, ftype
         else:
-            raise ValueError('either dest or ftype needs to be specified')
+            raise ValueError("either dest or ftype needs to be specified")
+        if not destext:
+            raise ValueError("can't determine the destination file type")
         # Sanitize destination file extension name.
         destext = destext.lower()
         # Set to self.
+        #: Relative directory of the source file.
         self.sourcedir = sourcedir
+        #: Absolute directory of the source file.
         self.sourcedirabs = sourcedirabs
+        #: Base name of the source file.
         self.sourcebase = sourcebase
+        #: Main part of the base name of the source file.
         self.sourcemain = sourcemain
+        #: Relative directory of the destination file.
         self.destdir = destdir
+        #: Absolute directory of the destination file.
         self.destdirabs = destdirabs
+        #: Main part of the base name of the destination file.
         self.destmain = destmain
+        #: Extension part of the base name of the destination file.  It is
+        #: always lower-case.
         self.destext = destext
-        # Temporary directory.
-        self._tempdir = None
+        # See :py:attr:`tempdir`.
+        self._tempdir = tempdir
 
     def __repr__(self):
         return "%s(source='%s', dest='%s')" % (
@@ -200,15 +247,23 @@ class Filename(object):
     @property
     def tempdir(self):
         """
-        Return a temporary directory.  If it doesn't exist yet, create one.
+        Return a temporary directory.  If it doesn't exist, create a new one.
+
+        The constructor can specify an existing directory as the temporary
+        directory.
+
+        Note: caller is responsible for deleting the created temporary
+        directory.
         """
-        if None is self._tempdir:
+        if None is self._tempdir or not os.path.isdir(self._tempdir):
             self._tempdir = tempfile.mkdtemp()
         return self._tempdir
 
     @property
     def source(self):
         """
+        The source file name.  It is the same as :py:attr:`sourcebase`.
+
         >>> Filename(source='dir1/name.tex', dest='dir2/', ftype='png').source
         'name.tex'
         """
@@ -217,6 +272,8 @@ class Filename(object):
     @property
     def dest(self):
         """
+        The destination file name.
+
         >>> Filename(source='dir1/name.tex', dest='dir2/', ftype='png').dest
         'name.png'
         >>> Filename(source='dir1/name.tex', dest='dir2/', ftype='eps').dest
@@ -227,36 +284,57 @@ class Filename(object):
     @property
     def sourcepath(self):
         """
-        >>> fn = Filename(source='dir1/name.tex', dest='dir2/', ftype='png')
-        >>> fn.sourcepath.endswith(fn.source)
-        True
+        The absolute path to the source file.
         """
         return os.path.join(self.sourcedirabs, self.source)
 
     @property
     def destpath(self):
         """
-        >>> fn = Filename(source='dir1/name.tex', dest='dir2/', ftype='png')
-        >>> fn.destpath.endswith(fn.dest)
-        True
+        The absolute path to the destination file.
         """
         return os.path.join(self.destdirabs, self.dest)
 
+    @property
+    def sourceext(self):
+        """
+        Extension part of the base name of the destination file.
+        """
+        return "tex"
+
     @property
     def ftype(self):
         """
-        >>> Filename(source='dir1/name.tex', dest='dir2/', ftype='png').ftype
-        'png'
-        >>> Filename(source='dir1/name.tex', dest='dir2/other.png').ftype
+        Type of the destination file.
+        
+        It should be the same as :py:attr:`destext`.
+
+        >>> Filename(source='dir1/name.tex', dest='name.png').ftype
         'png'
         """
         return self.destext
 
+    @property
+    def intertex(self):
+        """
+        The intermediate TEX file name for :py:meth:`Pstricks.pst`.
+
+        >>> Filename(source='dir1/name.tex', ftype='png').intertex
+        'name.tex'
+        >>> Filename(source='dir1/name.tex', dest='dir2/other.png').intertex
+        'other.tex'
+        """
+        return '.'.join([self.destmain, 'tex'])
+
     @property
     def intereps(self):
         """
-        >>> Filename(source='dir1/name.tex', dest='dir2/', ftype='png').intereps
+        The intermediate EPS file name for :py:meth:`Pstricks.pst`.
+
+        >>> Filename(source='dir1/name.tex', ftype='png').intereps
         'name.eps'
+        >>> Filename(source='dir1/name.tex', dest='dir2/other.png').intereps
+        'other.eps'
         """
         return '.'.join([self.destmain, 'eps'])
 
@@ -269,9 +347,12 @@ def main():
                         help='source file name')
     parser.add_argument('dst', type=str, nargs='?', default=None,
                         help='destination file name')
+    parser.add_argument('--font', dest='font', action='store',
+                        default=None, choices=[None, 'cmbright'],
+                        help='font selection (default = %(default)s)')
     parser.add_argument('-k', dest='keep_tmp', action='store_true',
                         default=False,
-                        help='keep temporary files')
+                        help='keep temporary directory')
     parser.add_argument('-t', dest='dstext', type=str, action='store',
                         default='png',
                         help='destination file type '
@@ -280,11 +361,17 @@ def main():
                         type=int, default=3,
                         help='verbosity of output (1-5) '
                              '(default = %(default)d)') # FIXME: not used.
+    parser.add_argument('--tempdir', dest='tempdir', action='store',
+                        default=None,
+                        help='DANGEROUS! Be sure to set -k, '
+                        'otherwise the temporary directory you specified '
+                        'will be deleted after the script ends.')
     args = parser.parse_args()
 
     runner = Pstricks(**vars(args))
-    fn = Filename(source=args.src, dest=args.dst, ftype=args.dstext)
-    runner(fn)
+    fn = Filename(source=args.src, dest=args.dst, ftype=args.dstext,
+                  tempdir=args.tempdir)
+    runner(fn, **vars(args))
 
     return 0
 
